/*
 *  Ray Trace Mandlebulb using CPU
 *  '0' snaps angles to 0,0
 *  arrows to rotate the world
 *  wasd   move light
 *  +/-    adjust power
 *  []     adjust iterations
 */

using namespace std;

#define GL_GLEXT_PROTOTYPES
#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif
#include <vector>

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include "Ray.h"
#include "MB.h"

//  Global variables
int th=0;                   //  Azimuth of view angle
int ph=0;                   //  Elevation of view angle
int Th=30;                  //  Azimuth of light angle
int Ph=30;                  //  Elevation of light angle
unsigned char* pixels=NULL; //  Pixel array
MandelBulb mb;

/*
 *   Mandeblub ray tracing code
 */
#define  DEVICE
#include "mandelbulb.h"

/*
 *  OpenGL (GLUT) calls this routine to display the scene
 */
void display()
{
   int k;
   double t = 0.001*glutGet(GLUT_ELAPSED_TIME);
   //  Set light direction
   mb.L.x = Sin(Th)*Cos(Ph);
   mb.L.y =         Sin(Ph);
   mb.L.z = Cos(Th)*Cos(Ph);
   //  Ray trace scene
   #pragma omp parallel for
   for (k=0;k<mb.hgt*mb.wid;k++)
      RayTracePixel(pixels,k,mb);
   //  Time ray tracing
   t = 0.001*glutGet(GLUT_ELAPSED_TIME) - t;
   //  Blit scene to screen
   glWindowPos2i(0,0);
   glDrawPixels(mb.wid,mb.hgt,GL_RGBA,GL_UNSIGNED_BYTE,pixels);
   //  Display
   glWindowPos2i(5,5);
   Print("Size %dx%d Time %.3fs Angle %d,%d Power %d MaxIter %d",mb.wid,mb.hgt,t,th,ph,mb.n,mb.maxiter);
   //  Flush
   glFlush();
   glutSwapBuffers();
}

/*
 *  Start up GLUT and tell it what to do
 */
int main(int argc,char* argv[])
{
   //  Initialize GLUT and process user parameters
   glutInit(&argc,argv);
   //  Request double buffered, true Color window with Z buffering at 600x600
   glutInitWindowSize(800,600);
   glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
   //  Create the window
   glutCreateWindow("Mandelbulb - CPU");
   //  Set GLUT callbacks
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   glutSpecialFunc(special);
   glutKeyboardFunc(key);
   //  Initialize Mandelbulb
   Init(mb);
   //  Pass control to GLUT so it can interact with the user
   glutMainLoop();
   return 0;
}
